""" Test MIP app functionality """

from subprocess import CalledProcessError

import pytest

from cg.constants import SINGLE_QUOTE, SPACE


def test_build_command(mip_api, mip_config_path, case_id):
    """ Test building a command """
    # GIVEN a config

    # WHEN building the MIP command
    mip_command = mip_api.build_command(case=case_id, config=mip_config_path)
    expected_binary_command = [mip_api.script, mip_api.pipeline, case_id]

    # THEN it should use the correct options
    assert mip_command == {
        "binary": SPACE.join(expected_binary_command),
        "environment": mip_api.conda_env,
        "config": mip_config_path,
        "parameters": [],
    }


def test_execute_when_dry_run(mip_api, mip_config_path: str, case_id: str):
    """Test executing a command"""
    ## GIVEN a config and a build command
    build_command = mip_api.build_command(case=case_id, config=mip_config_path)

    ## WHEN executing in dry run mode
    return_code = mip_api.execute(build_command, dry_run=True)

    ## THEN return success
    assert return_code == 0



"""Fixtures for the meta tests"""

from datetime import datetime

import pytest
from _pytest import tmpdir
from cg.apps.balsamic.fastq import FastqHandler as BalsamicFastqHandler
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.scout.scoutapi import ScoutAPI
from cg.meta.workflow.mip import MipAnalysisAPI
from cg.store import Store
from tests.store_helpers import StoreHelpers


@pytest.fixture(scope="function", name="analysis_store")
def fixture_analysis_store(base_store: Store, analysis_family: dict) -> Store:
    """Setup a store instance for testing analysis API."""
    customer = base_store.customer("cust000")
    family = base_store.Family(
        data_analysis=analysis_family["data_analysis"],
        name=analysis_family["name"],
        panels=analysis_family["panels"],
        internal_id=analysis_family["internal_id"],
        priority="standard",
    )
    family.customer = customer
    base_store.add(family)
    application_version = base_store.application("WGTPCFC030").versions[0]
    for sample_data in analysis_family["samples"]:
        sample = base_store.add_sample(
            name=sample_data["name"],
            sex=sample_data["sex"],
            internal_id=sample_data["internal_id"],
            ticket=sample_data["ticket_number"],
            reads=sample_data["reads"],
            capture_kit=sample_data["capture_kit"],
        )
        sample.family = family
        sample.application_version = application_version
        sample.customer = customer
        base_store.add(sample)
    base_store.commit()
    for sample_data in analysis_family["samples"]:
        sample_obj = base_store.sample(sample_data["internal_id"])
        link = base_store.relate_sample(
            family=family,
            sample=sample_obj,
            status=sample_data["status"],
            father=base_store.sample(sample_data["father"]) if sample_data.get("father") else None,
            mother=base_store.sample(sample_data["mother"]) if sample_data.get("mother") else None,
        )
        base_store.add(link)
    base_store.commit()
    yield base_store


class MockLims:
    """Mock lims fixture"""

    lims = None

    def __init__(self):
        self.lims = self


class MockPath:
    def __init__(self, path):
        self.yaml = None

    def __call__(self, *args, **kwargs):
        return self

    def open(self):
        return dict()

    def joinpath(self, path):
        return path


class MockLogger:
    warnings = []

    def warning(self, text, *interpolations):

        self.warnings.append(text % interpolations)

    def get_warnings(self) -> list:
        return self.warnings


class MockTB:
    _get_trending_raises_keyerror = False

    def __init__(self):
        """Needed to initialise mock variables"""
        self._make_config_was_called = False

    def get_trending(self, mip_config_raw: dict, qcmetrics_raw: dict, sampleinfo_raw: dict) -> dict:
        if self._get_trending_raises_keyerror:
            raise KeyError("mockmessage")

        # Returns: dict: parsed data
        ### Define output dict
        outdata = {
            "analysis_sex": {"ADM1": "female", "ADM2": "female", "ADM3": "female"},
            "case": "yellowhog",
            "duplicates": {"ADM1": 13.525, "ADM2": 12.525, "ADM3": 14.525},
            "mapped_reads": {"ADM1": 98.8, "ADM2": 99.8, "ADM3": 97.8},
            "mip_version": "v4.0.20",
            "sample_ids": ["2018-20203", "2018-20204"],
            "genome_build": "37",
            "rank_model_version": "1.18",
        }

        return outdata

    def get_sampleinfo(self, analysis_obj):
        return ""

    def make_config(self, data, pipeline):
        """Mock the make_config"""
        self._make_config_was_called = True
        del pipeline
        return data


class MockBalsamicFastq(BalsamicFastqHandler):
    """Mock FastqHandler for analysis_api"""

    def __init__(self):
        super().__init__(config={"balsamic": {"root": tmpdir}})


@pytest.fixture(scope="function", name="mip_hk_store")
def fixture_mip_hk_store(
    helpers: StoreHelpers,
    real_housekeeper_api: HousekeeperAPI,
    timestamp: datetime,
    case_id: str,
) -> HousekeeperAPI:
    deliver_hk_bundle_data = {
        "name": case_id,
        "created": timestamp,
        "expires": timestamp,
        "files": [
            {
                "path": "tests/fixtures/apps/mip/dna/store/case_config.yaml",
                "archive": False,
                "tags": ["mip-config"],
            },
            {
                "path": "tests/fixtures/apps/mip/dna/store/case_qc_sample_info.yaml",
                "archive": False,
                "tags": ["sampleinfo"],
            },
            {
                "path": "tests/fixtures/apps/mip/case_qc_metrics.yaml",
                "archive": False,
                "tags": ["qcmetrics"],
            },
            {
                "path": "tests/fixtures/apps/mip/case_file.txt",
                "archive": False,
                "tags": ["case-tag"],
            },
            {
                "path": "tests/fixtures/apps/mip/sample_file.txt",
                "archive": False,
                "tags": ["sample-tag", "ADM1"],
            },
        ],
    }
    helpers.ensure_hk_bundle(real_housekeeper_api, deliver_hk_bundle_data, include=True)

    empty_deliver_hk_bundle_data = {
        "name": "case_missing_data",
        "created": timestamp,
        "expires": timestamp,
        "files": [
            {
                "path": "tests/fixtures/apps/mip/dna/store/empty_case_config.yaml",
                "archive": False,
                "tags": ["mip-config"],
            },
            {
                "path": "tests/fixtures/apps/mip/dna/store/empty_case_qc_sample_info.yaml",
                "archive": False,
                "tags": ["sampleinfo"],
            },
            {
                "path": "tests/fixtures/apps/mip/dna/store/empty_case_qc_metrics.yaml",
                "archive": False,
                "tags": ["qcmetrics"],
            },
            {
                "path": "tests/fixtures/apps/mip/case_file.txt",
                "archive": False,
                "tags": ["case-tag"],
            },
            {
                "path": "tests/fixtures/apps/mip/sample_file.txt",
                "archive": False,
                "tags": ["sample-tag", "ADM1"],
            },
        ],
    }
    helpers.ensure_hk_bundle(real_housekeeper_api, empty_deliver_hk_bundle_data, include=True)

    return real_housekeeper_api


@pytest.fixture(scope="function", name="analysis_api")
def fixture_analysis_api(
    analysis_store: Store, mip_hk_store: HousekeeperAPI, scout_api: ScoutAPI
) -> MipAnalysisAPI:
    """Setup an analysis API."""

    analysis_api = MipAnalysisAPI(
        db=analysis_store,
        hk_api=mip_hk_store,
        scout_api=scout_api,
        tb_api=MockTB(),
        lims_api=MockLims(),
        script="echo",
        pipeline="analyse rd_dna",
        conda_env="S_mip_rd-dna",
        root="/var/empty",
    )
    yield analysis_api



"""
    Tests for GenotypeAPI
"""

import logging
import pytest

from cg.apps.gt import GenotypeAPI
from cg.exc import CaseNotFoundError


def test_instantiate(genotype_config: dict):
    """Test to instantiate a genotype api"""
    # GIVEN a genotype api with a config

    # WHEN instantiating a genotype api
    genotype_api = GenotypeAPI(genotype_config)

    # THEN assert that the adapter was properly instantiated
    assert genotype_api.process.config == genotype_config["genotype"]["config_path"]
    assert genotype_api.process.binary == genotype_config["genotype"]["binary_path"]


def test_genotype_api_upload(genotype_api: GenotypeAPI, caplog):
    """Test to run the upload functionality in the genotype API"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a genotype api and a samples sex dictionary and a bcf_path
    sample_id = "a_sample_id"
    samples_sex = {sample_id: {"pedigree": "Female", "analysis": "Female"}}
    bcf_path = "path_to_file.bcf"

    # WHEN running the update command
    genotype_api.upload(bcf_path, samples_sex)

    # THEN assert that the correct information was communicated
    assert f"loading VCF genotypes for sample(s): {sample_id}" in caplog.text


def test_update_sample_sex(genotype_api: GenotypeAPI, caplog):
    """Test to update the sample sex function"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a sample id and a sex
    sample = "sample"
    sex = "female"

    # WHEN running the update sample sex command
    genotype_api.update_sample_sex(sample, sex)

    # THEN assert that the correct message was logged
    assert f"Set sex for sample {sample} to {sex}"


def test_update_analysis_sex(genotype_api: GenotypeAPI, caplog):
    """Test to update the sample sex function"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a sample id and a sex
    sample = "sample"
    sex = "female"

    # WHEN running the update sample sex command
    genotype_api.update_analysis_sex(sample, sex)

    # THEN assert that the correct message was logged
    assert f"Set predicted sex for sample {sample} to {sex} for the sequence analysis"


def test_export_sample(genotype_api: GenotypeAPI, genotype_export_sample_output: str, caplog):
    """Test that get_trending calls the genotype API with correct command."""
    caplog.set_level(logging.DEBUG)
    # GIVEN a genotype api and argument days
    days = 20
    # GIVEN that the process returns some output
    genotype_api.process.stdout = genotype_export_sample_output

    # WHEN running get_trending
    genotype_api.export_sample(days=days)

    # THEN assert subprocess is running the GenotypeAPI with correct command
    call = ["config/path", "export-sample", "-d", str(days)]
    assert " ".join(call) in caplog.text


def test_export_sample_no_output(genotype_api: GenotypeAPI, caplog):
    """Test to get genotype via the api"""

    # GIVEN a genotype api
    # GIVEN a process that does not return any output

    # WHEN export_sample is calling the GenotypeAPI witch is returning a empty string

    # THEN assert CaseNotFoundError
    with pytest.raises(CaseNotFoundError):
        genotype_api.export_sample(days="")


def test_export_sample_analysis(
    genotype_api: GenotypeAPI, genotype_export_sample_analysis_output: str, caplog
):

    """Test that get_trending calls the genotype API with correct command."""
    caplog.set_level(logging.DEBUG)
    # GIVEN a genotype api and argument days
    days = 20
    # GIVEN that the process returns some output
    genotype_api.process.stdout = genotype_export_sample_analysis_output

    # WHEN running get_trending
    genotype_api.export_sample_analysis(days=days)

    # THEN assert subprocess is running the GenotypeAPI with correct command
    call = ["config/path", "export-sample-analysis", "-d", str(days)]
    print(genotype_api)
    assert " ".join(call) in caplog.text


def test_export_sample_analysis_no_output(genotype_api: GenotypeAPI):
    """Test to get case data via the api"""

    # GIVEN a genotype api
    # GIVEN a process that does not return any output

    # WHEN export_sample is calling the GenotypeAPI witch is returning a empty string

    # THEN assert CaseNotFoundError
    with pytest.raises(CaseNotFoundError):
        genotype_api.export_sample_analysis(days=20)



"""Fixtures for cli clean tests"""

import datetime as dt
from pathlib import Path

import pytest
from cg.apps.balsamic.api import BalsamicAPI
from cg.apps.hermes.hermes_api import HermesApi
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.constants import Pipeline
from cg.meta.workflow.balsamic import BalsamicAnalysisAPI
from cg.store import Store


@pytest.fixture
def balsamic_clean_store(base_store: Store, timestamp_yesterday: dt.datetime, helpers) -> Store:
    store = base_store

    # Create textbook case for cleaning
    case_to_clean = helpers.add_case(
        store=store, internal_id="balsamic_case_clean", case_id="balsamic_case_clean"
    )
    sample_case_to_clean = helpers.add_sample(
        store,
        internal_id="balsamic_sample_clean",
        is_tumour=True,
        application_type="wgs",
        data_analysis=Pipeline.BALSAMIC,
    )
    helpers.add_relationship(store, case=case_to_clean, sample=sample_case_to_clean)

    helpers.add_analysis(
        store,
        case=case_to_clean,
        pipeline=Pipeline.BALSAMIC,
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )

    # Create textbook case not for cleaning
    case_to_not_clean = helpers.add_case(
        store=store, internal_id="balsamic_case_not_clean", case_id="balsamic_case_not_clean"
    )
    case_to_not_clean.action = "running"
    store.commit()

    sample_case_to_not_clean = helpers.add_sample(
        store,
        internal_id="balsamic_sample_not_clean",
        is_tumour=True,
        application_type="wgs",
        data_analysis="balsamic",
    )
    helpers.add_relationship(store, case=case_to_not_clean, sample=sample_case_to_not_clean)

    helpers.add_analysis(
        store,
        case=case_to_not_clean,
        pipeline="balsamic",
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )
    return store


@pytest.fixture(name="balsamic_dir")
def balsamic_dir(tmpdir_factory, apps_dir: Path) -> Path:
    """Return the path to the balsamic apps dir"""
    balsamic_dir = tmpdir_factory.mktemp("balsamic")
    return Path(balsamic_dir).absolute().as_posix()


@pytest.fixture
def server_config(balsamic_dir: Path) -> dict:
    return {
        "database": "database",
        "bed_path": balsamic_dir,
        "balsamic": {
            "root": balsamic_dir,
            "singularity": Path(balsamic_dir, "singularity.sif").as_posix(),
            "reference_config": Path(balsamic_dir, "reference.json").as_posix(),
            "binary_path": "/home/proj/bin/conda/envs/S_BALSAMIC/bin/balsamic",
            "conda_env": "S_BALSAMIC",
            "slurm": {
                "mail_user": "test.mail@scilifelab.se",
                "account": "development",
                "qos": "low",
            },
        },
        "housekeeper": {
            "database": "database",
            "root": balsamic_dir,
        },
        "lims": {
            "host": "example.db",
            "username": "testuser",
            "password": "testpassword",
        },
    }


@pytest.fixture
def balsamic_analysis_api(
    server_config: dict,
    balsamic_clean_store: Store,
    housekeeper_api: HousekeeperAPI,
    hermes_api: HermesApi,
    trailblazer_api,
):
    return BalsamicAnalysisAPI(
        balsamic_api=BalsamicAPI(server_config),
        store=balsamic_clean_store,
        housekeeper_api=housekeeper_api,
        fastq_handler="FastqHandler",
        lims_api="LIMS",
        trailblazer_api=trailblazer_api,
        hermes_api=hermes_api,
    )


@pytest.fixture
def clean_context(
    base_store: Store,
    housekeeper_api: HousekeeperAPI,
    balsamic_analysis_api: BalsamicAnalysisAPI,
    helpers,
    tmpdir,
) -> dict:
    """context to use in cli"""

    return {
        "housekeeper_api": housekeeper_api,
        "store_api": balsamic_clean_store,
        "BalsamicAnalysisAPI": balsamic_analysis_api,
    }



"""This script tests the cli method to clean old balsamic run dirs"""
import datetime as dt
import logging
from datetime import datetime, timedelta
from pathlib import Path

from cg.cli.clean import balsamic_past_run_dirs, balsamic_run_dir
from cg.constants import Pipeline

EXIT_SUCCESS = 0


def test_past_run_dirs_without_options(cli_runner, clean_context: dict):
    """Test command without options"""
    # GIVEN no case_id or options
    # WHEN dry running without anything specified
    result = cli_runner.invoke(balsamic_past_run_dirs, obj=clean_context)
    # THEN command should mention argument
    assert result.exit_code != EXIT_SUCCESS
    assert "Missing argument" in result.output


def test_run_dir_without_options(cli_runner, clean_context: dict):
    """Test command without options"""
    # GIVEN no case_id or options
    # WHEN dry running without anything specified
    result = cli_runner.invoke(balsamic_run_dir, obj=clean_context)
    # THEN command should mention argument
    assert result.exit_code != EXIT_SUCCESS
    assert "Missing argument" in result.output


def test_with_yes(cli_runner, clean_context: dict, timestamp_today: dt.datetime, helpers, caplog):
    """Test command with dry run options"""
    # GIVEN a case on disk that could be deleted
    store = clean_context["BalsamicAnalysisAPI"].store
    timestamp_now = timestamp_today

    case_id = "balsamic_case_clean"
    analysis_to_clean = store.family(case_id).analyses[0]
    case_path = clean_context["BalsamicAnalysisAPI"].get_case_path(case_id)
    Path(case_path).mkdir()

    # WHEN running with yes and remove stuff from before today
    result = cli_runner.invoke(
        balsamic_past_run_dirs, ["-y", str(timestamp_now)], obj=clean_context
    )
    # THEN the analysis should have been cleaned
    assert result.exit_code == EXIT_SUCCESS
    assert analysis_to_clean.cleaned_at
    assert analysis_to_clean not in store.analyses_to_clean(pipeline=Pipeline.BALSAMIC)
    assert not Path(case_path).exists()


def test_dry_run(
    cli_runner, clean_context: dict, timestamp_yesterday: dt.datetime, helpers, caplog
):
    """Test command with dry run options"""

    # GIVEN a case on disk that could be deleted
    caplog.set_level(logging.INFO)
    base_store = clean_context["BalsamicAnalysisAPI"].store
    helpers.add_analysis(
        base_store,
        pipeline=Pipeline.BALSAMIC,
        started_at=timestamp_yesterday,
        uploaded_at=timestamp_yesterday,
        cleaned_at=None,
    )
    case_id = "balsamic_case_clean"
    analysis_to_clean = base_store.family(case_id).analyses[0]
    case_path = clean_context["BalsamicAnalysisAPI"].get_case_path(case_id)
    Path(case_path).mkdir()

    # WHEN dry running with dry run specified
    result = cli_runner.invoke(balsamic_run_dir, [case_id, "-d", "-y"], obj=clean_context)
    # THEN command should say it would have deleted
    assert result.exit_code == EXIT_SUCCESS
    assert "Would have deleted" in caplog.text
    assert case_id in caplog.text
    assert analysis_to_clean in base_store.analyses_to_clean(pipeline=Pipeline.BALSAMIC)


def test_cleaned_at_valid(cli_runner, clean_context: dict, caplog):
    """Test command with dry run options"""
    # GIVEN a case on disk that could be deleted
    base_store = clean_context["BalsamicAnalysisAPI"].store
    case_id = "balsamic_case_clean"
    case_path = clean_context["BalsamicAnalysisAPI"].get_case_path(case_id)
    Path(case_path).mkdir()
    # WHEN dry running with dry run specified
    result = cli_runner.invoke(balsamic_run_dir, [case_id, "-y"], obj=clean_context)

    # THEN command should say it would have deleted
    assert result.exit_code == EXIT_SUCCESS
    assert base_store.family("balsamic_case_clean").analyses[0].cleaned_at
    assert not Path(case_path).exists()


def test_cleaned_at_invalid(cli_runner, clean_context: dict, caplog):
    """Test command with dry run options"""
    # GIVEN a case on disk that could be deleted
    base_store = clean_context["BalsamicAnalysisAPI"].store
    case_id = "balsamic_case_not_clean"
    case_path = clean_context["BalsamicAnalysisAPI"].get_case_path(case_id)
    Path(case_path).mkdir()
    assert not base_store.family("balsamic_case_not_clean").analyses[0].cleaned_at
    # WHEN dry running with dry run specified

    result = cli_runner.invoke(
        balsamic_past_run_dirs, ["2020-12-01", "-d", "-y"], obj=clean_context
    )

    # THEN case directory should not have been cleaned
    assert result.exit_code == EXIT_SUCCESS
    assert not base_store.family("balsamic_case_not_clean").analyses[0].cleaned_at
    assert Path(case_path).exists()



import datetime

from cg.store import Store


def test_add_sample_bad_customer(invoke_cli, disk_store: Store):
    # GIVEN an empty database

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = "dummy_application"
    customer_id = "dummy_customer"
    name = "dummy_name"
    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            customer_id,
            name,
        ]
    )

    # THEN then it should complain about missing customer instead of adding a sample
    assert result.exit_code == 1
    assert disk_store.Sample.query.count() == 0


def test_add_sample_bad_application(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = "dummy_application"
    customer_id = add_customer(disk_store)
    name = "dummy_name"
    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            customer_id,
            name,
        ]
    )

    # THEN then it should complain about missing application instead of adding a sample
    assert result.exit_code == 1
    assert disk_store.Sample.query.count() == 0


def test_add_sample_required(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer and an application

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = add_application(disk_store)
    customer_id = add_customer(disk_store)
    name = "sample_name"

    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            customer_id,
            name,
        ]
    )

    # THEN then it should be added
    assert result.exit_code == 0
    assert disk_store.Sample.query.count() == 1
    assert disk_store.Sample.query.first().name == name
    assert disk_store.Sample.query.first().sex == sex


def test_add_sample_lims_id(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer and an application

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = add_application(disk_store)
    customer_id = add_customer(disk_store)
    name = "sample_name"
    lims_id = "sample_lims_id"

    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            "--lims",
            lims_id,
            customer_id,
            name,
        ]
    )

    # THEN then it should be added
    assert result.exit_code == 0
    assert disk_store.Sample.query.count() == 1
    assert disk_store.Sample.query.first().internal_id == lims_id


def test_add_sample_order(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer and an application

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = add_application(disk_store)
    customer_id = add_customer(disk_store)
    name = "sample_name"
    order = "sample_order"

    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            "--order",
            order,
            customer_id,
            name,
        ]
    )

    # THEN then it should be added
    assert result.exit_code == 0
    assert disk_store.Sample.query.count() == 1
    assert disk_store.Sample.query.first().order == order


def test_add_sample_downsampled(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer and an application

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = add_application(disk_store)
    customer_id = add_customer(disk_store)
    name = "sample_name"
    downsampled_to = "123"

    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            "--downsampled",
            downsampled_to,
            customer_id,
            name,
        ]
    )

    # THEN then it should be added
    assert result.exit_code == 0
    assert disk_store.Sample.query.count() == 1
    assert str(disk_store.Sample.query.first().downsampled_to) == downsampled_to


def test_add_sample_priority(invoke_cli, disk_store: Store):
    # GIVEN a database with a customer and an application

    # WHEN adding a sample
    db_uri = disk_store.uri
    sex = "male"
    application = add_application(disk_store)
    customer_id = add_customer(disk_store)
    name = "sample_name"
    priority = "priority"

    result = invoke_cli(
        [
            "--database",
            db_uri,
            "add",
            "sample",
            "--sex",
            sex,
            "--application",
            application,
            "--priority",
            priority,
            customer_id,
            name,
        ]
    )

    # THEN then it should be added
    assert result.exit_code == 0
    assert disk_store.Sample.query.count() == 1
    assert disk_store.Sample.query.first().priority_human == priority


def add_customer(disk_store, customer_id="cust_test"):
    customer_group = disk_store.add_customer_group("dummy_group", "dummy group")
    customer = disk_store.add_customer(
        internal_id=customer_id,
        name="Test Customer",
        scout_access=False,
        customer_group=customer_group,
        invoice_address="dummy_address",
        invoice_reference="dummy_reference",
    )
    disk_store.add_commit(customer)
    return customer_id


def add_application(disk_store, application_tag="dummy_tag"):

    application = disk_store.add_application(
        tag=application_tag,
        category="wgs",
        description="dummy_description",
        percent_kth=80,
        percent_reads_guaranteed=75,
    )
    disk_store.add_commit(application)
    prices = {"standard": 10, "priority": 20, "express": 30, "research": 5}
    version = disk_store.add_version(
        application, 1, valid_from=datetime.datetime.now(), prices=prices
    )

    disk_store.add_commit(version)
    return application_tag



""" Test the CLI for run mip-dna """

import logging

from cg.meta.workflow.mip import MipAnalysisAPI
from cg.meta.workflow.prepare_fastq import PrepareFastqAPI
from cg.cli.workflow.mip_dna.base import start, start_available
from cg.store.api.status import StatusHandler

from tests.meta.conftest import fixture_analysis_api
from tests.meta.conftest import fixture_mip_hk_store


def test_spring_decompression_needed_and_started(
    mocker, cli_runner, caplog, dna_mip_context, case_id, analysis_api
):
    """Tests starting the MIP analysis when decompression is needed"""
    caplog.set_level(logging.INFO)

    # GIVEN a case to analyze
    mip_api = analysis_api
    case_object = mip_api.get_case_object(case_id)

    # GIVEN a case is available for analysis
    mocker.patch.object(StatusHandler, "cases_to_analyze")
    StatusHandler.cases_to_analyze.return_value = [case_object]

    # GIVEN all samples in the case has dna application type
    mocker.patch.object(MipAnalysisAPI, "is_dna_only_case")
    MipAnalysisAPI.is_dna_only_case.return_value = True

    case_object.internal_id = "ADM1"

    # GIVEN the latest analysis has not started
    mocker.patch.object(MipAnalysisAPI, "has_latest_analysis_started")
    MipAnalysisAPI.has_latest_analysis_started.return_value = False

    # GIVEN spring decompression is needed
    mocker.patch.object(PrepareFastqAPI, "is_spring_decompression_needed")
    PrepareFastqAPI.is_spring_decompression_needed.return_value = True

    # GIVEN spring decompression starts successfully
    mocker.patch.object(PrepareFastqAPI, "start_spring_decompression")
    PrepareFastqAPI.start_spring_decompression.return_value = True

    # WHEN an MIP analysis is started
    result = cli_runner.invoke(start_available, obj=dna_mip_context)

    # THEN command should run without errors
    assert result.exit_code == 0

    # THEN it should be announced that spring decompression is started
    assert "started decompression instead" in caplog.text


def test_spring_decompression_needed_and_start_failed(
    mocker, cli_runner, caplog, dna_mip_context, case_id, analysis_api
):
    """Tests starting the MIP analysis when decompression is needed but can't start"""
    caplog.set_level(logging.INFO)

    # GIVEN a case to analyze
    mip_api = analysis_api
    case_object = mip_api.get_case_object(case_id)

    # GIVEN a case is available for analysis
    mocker.patch.object(StatusHandler, "cases_to_analyze")
    StatusHandler.cases_to_analyze.return_value = [case_object]

    # GIVEN all samples in the case has dna application type
    mocker.patch.object(MipAnalysisAPI, "is_dna_only_case")
    MipAnalysisAPI.is_dna_only_case.return_value = True

    case_object.internal_id = "ADM1"

    # GIVEN the latest analysis has not started
    mocker.patch.object(MipAnalysisAPI, "has_latest_analysis_started")
    MipAnalysisAPI.has_latest_analysis_started.return_value = False

    # GIVEN spring decompression is needed
    mocker.patch.object(PrepareFastqAPI, "is_spring_decompression_needed")
    PrepareFastqAPI.is_spring_decompression_needed.return_value = True

    # GIVEN spring decompression fail to start
    mocker.patch.object(PrepareFastqAPI, "start_spring_decompression")
    PrepareFastqAPI.start_spring_decompression.return_value = False

    # WHEN an MIP analysis is started
    result = cli_runner.invoke(start_available, obj=dna_mip_context)

    # THEN command should run without errors
    assert result.exit_code == 0

    # THEN it should be announced that spring decompression is needed but fail to start
    assert f"Decompression is needed but could not be started for" in caplog.text


def test_case_needs_to_be_stored(
    mocker, cli_runner, caplog, case_id, analysis_api, dna_mip_context
):
    """Test starting MIP when files are decompressed but not stored in housekeeper"""
    caplog.set_level(logging.INFO)

    # GIVEN a case to analyze
    mip_api = analysis_api
    case_object = mip_api.get_case_object(case_id)

    # GIVEN a case is available for analysis
    mocker.patch.object(StatusHandler, "cases_to_analyze")
    StatusHandler.cases_to_analyze.return_value = [case_object]

    # GIVEN all samples in the case has dna application type
    mocker.patch.object(MipAnalysisAPI, "is_dna_only_case")
    MipAnalysisAPI.is_dna_only_case.return_value = True

    case_object.internal_id = "ADM1"

    # GIVEN the latest analysis has not started
    mocker.patch.object(MipAnalysisAPI, "has_latest_analysis_started")
    MipAnalysisAPI.has_latest_analysis_started.return_value = False

    # GIVEN spring decompression is not needed
    mocker.patch.object(PrepareFastqAPI, "is_spring_decompression_needed")
    PrepareFastqAPI.is_spring_decompression_needed.return_value = False

    # GIVEN fastqs linked in housekeeper are checked successfully
    mocker.patch.object(PrepareFastqAPI, "check_fastq_links")
    PrepareFastqAPI.check_fastq_links.return_value = None

    # GIVEN spring decompression is not running
    mocker.patch.object(PrepareFastqAPI, "is_spring_decompression_running")
    PrepareFastqAPI.is_spring_decompression_running.return_value = False

    # GIVEN a panel file is created
    mocker.patch.object(MipAnalysisAPI, "panel")
    MipAnalysisAPI.panel.return_value = "bla"

    # WHEN MIP analysis is started
    result = cli_runner.invoke(
        start, ["ADM1", "--panel-bed", "panel.bed", "--dry-run"], obj=dna_mip_context
    )

    # THEN command should run without errors
    assert result.exit_code == 0

    # THEN fastq files should be added to housekeeper
    assert "Linking fastq files in housekeeper for case" in caplog.text



"""
Tests for command module
"""
from subprocess import CalledProcessError

import pytest
from cg.utils import Process


def test_process():
    # GIVEN a binary with 'ls'
    binary = "ls"
    # WHEN instantiating a Process with the binary
    process = Process(binary=binary)
    # THEN assert the instantiation worked as expected
    assert process.binary == binary
    assert repr(process) == "Process:base_call:['ls']"
    assert process.stdout == ""


def test_process_run_invalid_command():
    # GIVEN a binary with non existing command
    binary = "false"
    process = Process(binary=binary)
    # WHEN running the command
    with pytest.raises(CalledProcessError):
        # THEN assert an exception is raised
        process.run_command()


def test_process_run_command_no_params(ls_process):
    # GIVEN a proces with 'ls' as binary
    process = ls_process
    # WHEN running the command without parameters
    process.run_command()
    # THEN assert stdout is captures in self.stdout
    assert "README" in process.stdout


def test_process_run_command_with_params(ls_process):
    # GIVEN a process with 'ls' as binary
    process = ls_process
    # WHEN running the command with parameters
    process.run_command(parameters=["-l"])
    # THEN assert the output is valid
    i = 0
    for i, line in enumerate(process.stdout.split("\n"), 1):
        if i > 1:
            assert len(line.split()) >= 9
    assert i > 1


def test_process_std_err(ls_process):
    # GIVEN a proces with 'ls' as binary
    process = ls_process
    # WHEN running the command with invalid parameters
    with pytest.raises(CalledProcessError):
        # THEN assert that a exception is raised
        process.run_command(parameters=["-kffd4"])
    # THEN assert that stderr was captured
    assert process.stderr


def test_iter_output_lines(ls_process):
    # GIVEN a proces with 'ls' as binary
    process = ls_process
    # WHEN running the command without parameters
    process.run_command()
    nr_output_lines = 0
    for nr_output_lines, _line in enumerate(process.stdout_lines(), 1):
        pass
    # THEN assert that the method returns some lines
    assert nr_output_lines > 0


def test_iter_stderr_lines(ls_process, stderr_output):
    # GIVEN a proces with some stderr output
    process = ls_process
    process.stderr = stderr_output
    # WHEN running the command without parameters
    # THEN assert there where lines to iterate
    nr_stderr_lines = 0
    for nr_stderr_lines, _line in enumerate(process.stderr_lines(), 1):
        pass
    assert nr_stderr_lines > 0


def test_iter_stderr_lines_no_output(ls_process):
    # GIVEN a proces with some stderr output
    process = ls_process
    # WHEN running the command without parameters
    # THEN assert there where lines to iterate
    i = 0
    for i, line in enumerate(process.stderr_lines(), 1):
        assert line == ""
    assert i == 1



from cg.apps.lims import limsjson
from cg.constants import DataDelivery, Pipeline


def test_parsing_balsamic_json(balsamic_order_to_submit):

    # GIVEN an order form for a cancer order with 11 samples,
    # WHEN parsing the order form
    parsed_order = limsjson.parse_json_order(balsamic_order_to_submit)

    # THEN it should detect the type of project
    assert parsed_order["project_type"] == str(Pipeline.BALSAMIC)

    # ... and it should find and group all samples in case
    assert len(parsed_order["items"]) == 1

    # ... and collect relevant data about the case
    # ... and collect relevant info about the samples

    case = parsed_order["items"][0]
    sample = case["samples"][0]
    assert len(case["samples"]) == 1

    # This information is required

    assert sample["name"] == "s1"
    assert sample["container"] == "96 well plate"
    assert sample["data_analysis"].lower() == str(Pipeline.BALSAMIC)
    assert sample["data_delivery"] == str(DataDelivery.SCOUT)
    assert sample["application"] == "WGTPCFC030"
    assert sample["sex"] == "male"
    assert case["name"] == "family1"
    assert parsed_order["customer"] == "cust000"
    assert case["require_qcok"] is False
    assert sample["source"] == "blood"
    assert case["priority"] == "standard"

    # Required if Plate
    assert sample["container_name"] == "p1"
    assert sample["well_position"] == "A:1"

    # This information is required for panel- or exome analysis
    assert sample["elution_buffer"] == "Other (specify in 'Comments')"

    # This information is required for Balsamic analysis (cancer)
    assert sample["tumour"] is True
    assert sample["capture_kit"] == "LymphoMATIC"
    assert sample["tumour_purity"] == "75"

    assert sample["formalin_fixation_time"] == "1"
    assert sample["post_formalin_fixation_time"] == "2"
    assert sample["tissue_block_size"] == "small"

    # This information is optional
    assert sample["quantity"] == "2"
    assert sample["comment"] == "other Elution buffer"


def test_parsing_rml_json(rml_order_to_submit: dict) -> None:
    # GIVEN a path to a RML limsjson with 2 sample in a pool

    # WHEN parsing the file
    parsed_order = limsjson.parse_json_order(rml_order_to_submit)

    # THEN it should determine the type of project and customer
    assert parsed_order["project_type"] == "fluffy"
    assert parsed_order["customer"] == "cust000"
    assert parsed_order["comment"] == "order comment"

    # ... and find all samples
    assert len(parsed_order["items"]) == 4

    # ... and collect relevant sample data
    order_sample = parsed_order["items"][2]
    assert order_sample["name"] == "sample3"
    assert order_sample["pool"] == "pool-2"
    assert order_sample["application"] == "RMLS05R150"
    assert order_sample["data_analysis"] == str(Pipeline.FLUFFY)
    assert order_sample["data_delivery"] == str(DataDelivery.NIPT_VIEWER)
    assert order_sample["volume"] == "30"
    assert order_sample["concentration"] == "5"
    assert order_sample["index"] == "IDT DupSeq 10 bp Set B"
    assert order_sample["index_number"] == "3"
    assert order_sample.get("container_name") is None
    assert order_sample["rml_plate_name"] == "plate1"
    assert order_sample.get("well_position") is None
    assert order_sample["well_position_rml"] == "A:1"
    assert order_sample["index_sequence"] == "A01 - D701-D501 (ATTACTCG-TATAGCCT)"
    assert order_sample["comment"] == "test comment"
    assert order_sample["concentration_sample"] == "6"


def test_parsing_mip_json(mip_json_order_to_submit: dict):

    # GIVEN an order form for a Scout order with samples, 1 trio, in a plate
    # WHEN parsing the order form
    parsed_order = limsjson.parse_json_order(mip_json_order_to_submit)

    # THEN it should detect the type of project
    assert parsed_order["project_type"] == "mip-dna"
    assert parsed_order["customer"] == "cust000"

    # ... and it should find and group all samples in families
    assert len(parsed_order["items"]) == 7

    # ... and collect relevant data about the families
    duo_family = parsed_order["items"][4]
    assert len(duo_family["samples"]) == 2
    assert duo_family["name"] == "F0018192"
    assert duo_family["priority"] == "standard"
    assert set(duo_family["panels"]) == set(["CTD"])
    assert duo_family["require_qcok"] is False

    # ... and collect relevant info about the samples
    sibling_sample = duo_family["samples"][0]
    assert sibling_sample["name"] == "2020-16171-81"
    assert sibling_sample["application"] == "WGSPCFC030"
    assert sibling_sample["sex"] == "female"

    # case-id on the case
    # customer on the order (parsed_order)
    # require-qc-ok on the case
    assert sibling_sample["source"] == "blood"
    assert sibling_sample.get("tumour") is None
    assert sibling_sample["container"] == "96 well plate"
    assert sibling_sample["container_name"] == "2020-16171-81"
    assert sibling_sample["well_position"] == "A:1"

    # panels on the case
    assert sibling_sample["status"] == "affected"
    assert sibling_sample["mother"] == ""
    assert sibling_sample["father"] == ""
    assert sibling_sample["quantity"] == ""
    assert sibling_sample["comment"] == "F0018192-0 Data finns 2019-19202 duo med bror "

    # cohort
    assert sibling_sample["cohorts"] == ["Other"]

    # synopsis
    assert sibling_sample["synopsis"] == [
        "H\u00e4r kommer det att komma en v\u00e4ldigt l\u00e5ng text med f\u00f6r synopsis."
    ]

    # hpo-terms
    assert sibling_sample["phenotype_terms"] == ["HP:0012747", "HP:0025049"]

    # age at sampling
    assert sibling_sample["age_at_sampling"] == "17.18192"



"""Base command for trending"""

import logging
from pathlib import Path

import click
from cg.apps.gt import GenotypeAPI
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.vogue import VogueAPI
from cg.cli.workflow.get_links import get_links
from cg.constants import Pipeline
from cg.exc import AnalysisUploadError
from cg.meta.upload.vogue import UploadVogueAPI
from cg.store import Store

LOG = logging.getLogger(__name__)

VOGUE_VALID_BIOINFO = [str(Pipeline.MIP_DNA), str(Pipeline.BALSAMIC)]


@click.group()
@click.pass_context
def vogue(context):
    """Load trending data into trending database"""

    click.echo(click.style("----------------- TRENDING -----------------------"))

    context.obj["status_db"] = Store(context.obj["database"])
    context.obj["vogue_api"] = VogueAPI(context.obj)
    context.obj["vogue_upload_api"] = UploadVogueAPI(
        genotype_api=GenotypeAPI(context.obj),
        vogue_api=context.obj["vogue_api"],
        store=context.obj["status_db"],
    )


@vogue.command("genotype", short_help="Getting genotype data from the genotype database.")
@click.option(
    "-d", "--days", type=int, required="True", help="load X days old sampels from genotype to vogue"
)
@click.pass_context
def genotype(context, days: int):
    """Loading samples from the genotype database to the trending database"""

    click.echo(click.style("----------------- GENOTYPE -----------------------"))

    context.obj["vogue_upload_api"].load_genotype(days=days)


@vogue.command("apptags", short_help="Getting application tags to the trending database.")
@click.pass_context
def apptags(context):
    """Loading apptags from status db to the trending database"""

    click.echo(click.style("----------------- APPLICATION TAGS -----------------------"))

    context.obj["vogue_upload_api"].load_apptags()


@vogue.command("flowcells", short_help="Getting flowcell data from the lims.")
@click.option(
    "-d", "--days", type=int, required="True", help="load X days old runs from lims to vogue"
)
@click.pass_context
def flowcells(context, days: int):
    """Loading runs from lims to the trending database"""

    LOG.info("----------------- FLOWCELLS -----------------------")

    context.obj["vogue_api"].load_flowcells(days=days)


@vogue.command("samples", short_help="Getting sample data from lims.")
@click.option(
    "-d", "--days", type=int, required="True", help="load X days old sampels from lims to vogue"
)
@click.pass_context
def samples(context, days: int):
    """Loading samples from lims to the trending database"""

    LOG.info("----------------- SAMPLES -----------------------")

    context.obj["vogue_api"].load_samples(days=days)


@vogue.command("reagent-labels", short_help="Getting reagent_label data from lims.")
@click.option(
    "-d", "--days", type=int, required=True, help="load X days old sampels from lims to vogue"
)
@click.pass_context
def reagent_labels(context, days: int):
    """Loading reagent_labels from lims to the trending database"""

    LOG.info("----------------- REAGENT LABELS -----------------------")

    context.obj["vogue_api"].load_reagent_labels(days=days)


@vogue.command("bioinfo", short_help="Load bioinfo results into vogue")
@click.option(
    "-c",
    "--case-name",
    required=True,
    help="Case name or project name for which the analysis results will load",
)
@click.option(
    "--cleanup/--no-cleanup", default=False, help="Cleanup processed case data while loading"
)
@click.option(
    "-t",
    "--target-load",
    default="all",
    type=click.Choice(["all", "raw", "process"]),
    show_default=True,
    help=(
        "Final target to load bioinfo data."
        "Target orders are: all, raw, process."
        "Selecting all, will load raw, process, and sample."
        "Selecting process, will upload raw and process."
        "Selecting raw, will only load raw data."
    ),
)
@click.option("--dry/--no-dry", default=False, help="Dry run...")
@click.pass_context
def bioinfo(context, case_name, cleanup, target_load, dry):
    """Load bioinfo case results to the trending database"""

    hk_api = context.obj["housekeeper_api"]
    store = context.obj["status_db"]

    click.echo(click.style("----------------- BIOINFO -----------------------"))

    load_bioinfo_raw_inputs = dict()

    # Probably get samples for a case_name through statusdb api
    load_bioinfo_raw_inputs["samples"] = _get_samples(store, case_name)

    # Probably get analysis result file through housekeeper ai
    load_bioinfo_raw_inputs["analysis_result_file"] = _get_multiqc_latest_file(hk_api, case_name)

    # Probably get analysis_type [multiqc or microsalt or all] from cli
    # This might automated to some extend by checking if input multiqc json.
    # This tells us how the result was generated. If it is multiqc it will try to validate keys with
    # an actual model.
    load_bioinfo_raw_inputs["analysis_type"] = "multiqc"

    # case_name is the input
    load_bioinfo_raw_inputs["analysis_case_name"] = case_name

    # Get case_analysis_type from cli a free text for an entry in trending database
    load_bioinfo_raw_inputs["case_analysis_type"] = "multiqc"

    # Get workflow_name and workflow_version
    workflow_name, workflow_version = _get_analysis_workflow_details(store, case_name)
    if workflow_name not in VOGUE_VALID_BIOINFO:
        raise AnalysisUploadError(f"Case upload failed: {case_name}. Reason: Bad workflow name.")
    load_bioinfo_raw_inputs["analysis_workflow_name"] = workflow_name
    load_bioinfo_raw_inputs["analysis_workflow_version"] = workflow_version

    if dry:
        click.echo(click.style("----------------- DRY RUN -----------------------"))

    if target_load in ("raw", "all"):
        click.echo(click.style("----------------- UPLOAD UNPROCESSED -----------------------"))
        if not dry:
            context.obj["vogue_upload_api"].load_bioinfo_raw(load_bioinfo_raw_inputs)

    if target_load in ("process", "all"):
        click.echo(click.style("----------------- PROCESS CASE -----------------------"))
        if not dry:
            context.obj["vogue_upload_api"].load_bioinfo_process(load_bioinfo_raw_inputs, cleanup)
        click.echo(click.style("----------------- PROCESS SAMPLE -----------------------"))
        if not dry:
            context.obj["vogue_upload_api"].load_bioinfo_sample(load_bioinfo_raw_inputs)


@vogue.command("bioinfo-all", short_help="Load all bioinfo results into vogue")
@click.option("--dry/--no-dry", is_flag=True, help="Dry run...")
@click.pass_context
def bioinfo_all(context, dry):
    """Load all cases with recent analysis and a multiqc-json to the trending database."""

    hk_api = context.obj["housekeeper_api"]
    store = context.obj["status_db"]
    cases = store.families()
    for case in cases:
        case_name = case.internal_id
        version_obj = hk_api.last_version(case_name)
        if not version_obj:
            continue

        # confirm multiqc.json exists
        multiqc_file_obj = hk_api.get_files(
            bundle=case_name, tags=["multiqc-json"], version=version_obj.id
        )
        if len(list(multiqc_file_obj)) == 0:
            continue

        # confirm that file exists
        existing_multiqc_file = multiqc_file_obj[0].full_path
        if not Path(existing_multiqc_file).exists():
            continue

        LOG.info("Found multiqc for %s, %s", case_name, existing_multiqc_file)
        try:
            context.invoke(bioinfo, case_name=case_name, cleanup=True, target_load="all", dry=dry)
        except AnalysisUploadError:
            LOG.error("Case upload failed: %s", case_name, exc_info=True)


def _get_multiqc_latest_file(hk_api: HousekeeperAPI, case_name: str) -> str:
    """Get latest multiqc_data.json path for a case_name
    Args:
        case_name(str): onemite
    Returns:
        multiqc_data_path(str): /path/to/multiqc.json
    """
    version_obj = hk_api.last_version(case_name)
    multiqc_json_file = hk_api.get_files(
        bundle=case_name, tags=["multiqc-json"], version=version_obj.id
    )

    if len(list(multiqc_json_file)) == 0:
        raise FileNotFoundError(f"No multiqc.json was found in housekeeper for {case_name}")

    return multiqc_json_file[0].full_path


def _get_samples(store: Store, case_name: str) -> str:
    """Get a sample string for case_name
    Args:
        case_name(str): onemite
    Returns:
        sample_names(str): ACC12345,ACC45679
    """

    link_objs = get_links(store, case_name)
    sample_ids = {link_obj.sample.internal_id for link_obj in link_objs}
    return ",".join(sample_ids)


def _get_analysis_workflow_details(status_api: Store, case_name: str) -> str:
    """Get lowercase workflow name for a case_name
    Args:
        case_name(str): onemite
    Returns:
        workflow_name(str): balsamic
        workflow_version(str): v3.14.15
    """
    # Workflow that generated these results
    case_obj = status_api.family(case_name)
    workflow_name = None
    workflow_version = None
    if case_obj.analyses:
        workflow_name = case_obj.analyses[0].pipeline
        workflow_version = case_obj.analyses[0].pipeline_version

    return workflow_name.lower(), workflow_version



from typing import List, Set

from cg.apps.lims.orderform import CASE_PROJECT_TYPES, expand_case
from cg.constants import DataDelivery, Pipeline
from cg.exc import OrderFormError
from cg.meta.orders import OrderType
from cg.meta.orders.status import StatusHandler

ACCEPTED_DATA_ANALYSES: List[str] = [
    str(Pipeline.MIP_DNA),
    str(Pipeline.FLUFFY),
    str(Pipeline.BALSAMIC),
]


def get_project_type(samples: [dict]) -> str:
    """Determine the project type."""

    data_analyses: Set[str] = set(
        sample.get("data_analysis", "mip-dna").lower() for sample in samples
    )

    if len(data_analyses) > 1:
        raise OrderFormError(f"mixed 'Data Analysis' types: {', '.join(data_analyses)}")

    data_analysis: str = data_analyses.pop()
    if data_analysis in ACCEPTED_DATA_ANALYSES:
        return data_analysis

    raise OrderFormError(f"Unsupported order_data orderform: {data_analyses}")


def get_data_delivery(samples: [dict], project_type: OrderType) -> str:
    """Determine the order_data delivery type."""

    NO_VALUE = "no_value"
    data_deliveries = set(sample.get("data_delivery", NO_VALUE).lower() for sample in samples)

    if len(data_deliveries) > 1:
        raise OrderFormError(f"mixed 'Data Delivery' types: {', '.join(data_deliveries)}")

    data_delivery = data_deliveries.pop()

    if data_delivery == NO_VALUE:
        if project_type == OrderType.METAGENOME:
            return str(DataDelivery.FASTQ)
        if project_type == OrderType.FASTQ:
            return str(DataDelivery.FASTQ)
        if project_type == OrderType.RML:
            return str(DataDelivery.FASTQ)
        if project_type == OrderType.MIP_RNA:
            return str(DataDelivery.ANALYSIS_FILES)
        if project_type == OrderType.FLUFFY:
            return str(DataDelivery.NIPT_VIEWER)

        return ""

    try:
        return str(DataDelivery(data_delivery))
    except ValueError:
        raise OrderFormError(f"Unsupported order_data delivery: {data_delivery}")


def parse_json_order(order_data: dict) -> dict:
    """Parse JSON from LIMS export."""

    samples = order_data.get("samples")

    if not samples:
        raise OrderFormError("orderform doesn't contain any samples")

    project_type = get_project_type(samples)
    data_delivery = get_data_delivery(samples, OrderType(project_type))
    customer_id = order_data["customer"].lower()
    comment = order_data.get("comment")
    order_name = order_data.get("name")

    if project_type in CASE_PROJECT_TYPES:
        parsed_cases = StatusHandler.group_cases(samples)
        items = []
        for case_id, parsed_case in parsed_cases.items():
            case_data = expand_case(case_id, parsed_case)
            items.append(case_data)
    else:
        items = samples

    parsed_order = {
        "comment": comment,
        "customer": customer_id,
        "delivery_type": str(data_delivery),
        "items": items,
        "name": order_name,
        "project_type": project_type,
    }

    return parsed_order


def expand_case(case_id: str, parsed_case: dict) -> dict:
    """Fill-in information about families."""
    new_case = {"name": case_id, "samples": []}
    samples = parsed_case

    require_qcoks = set(raw_sample["require_qcok"] for raw_sample in samples)
    new_case["require_qcok"] = True in require_qcoks

    priorities = set(raw_sample["priority"].lower() for raw_sample in samples)
    if len(priorities) != 1:
        raise OrderFormError(f"multiple values for 'Priority' for case: {case_id}")
    new_case["priority"] = priorities.pop()

    gene_panels = set()
    for raw_sample in samples:
        if raw_sample.get("panels"):
            gene_panels.update(raw_sample["panels"])

        new_sample = {}

        for key, value in raw_sample.items():
            if key not in ["panels", "well_position"]:
                new_sample[key] = value

        well_position_raw = raw_sample.get("well_position")
        if well_position_raw:
            new_sample["well_position"] = (
                ":".join(well_position_raw) if ":" not in well_position_raw else well_position_raw
            )

        new_case["samples"].append(new_sample)

    if gene_panels:
        new_case["panels"] = list(gene_panels)

    return new_case



import datetime
import os
from pathlib import Path

from cg.meta.report.api import ReportAPI


def test_collect_delivery_data(report_api, report_store, case_id):
    # GIVEN an initialised report_api and a case ready for delivery report creation

    # WHEN collecting delivery data for a certain case
    case = report_store.family(case_id)
    assert case
    assert case.links
    assert case.analyses
    delivery_data = report_api._get_delivery_data(case_id=case_id)

    # THEN all data for the delivery report should have been collected
    assert delivery_data["report_version"]
    assert delivery_data["previous_report_version"]
    assert delivery_data["case"]
    assert delivery_data["customer_name"]
    assert delivery_data["today"]
    assert delivery_data["panels"]

    assert delivery_data["customer_invoice_address"]
    assert delivery_data["customer_name"]
    assert delivery_data["scout_access"]
    assert delivery_data["accredited"]

    for application in delivery_data["applications"]:
        assert application["tag"]
        assert application["description"]
        assert application["limitations"]

    assert delivery_data["samples"]

    for sample in delivery_data["samples"]:
        assert sample["name"]
        assert sample["internal_id"]
        assert sample["sex"]
        assert sample["status"]
        assert sample["source"]
        assert sample["ticket"]
        assert sample["application"]
        assert sample["received_at"]

        assert sample["ordered_at"]
        assert sample["prep_method"]
        assert sample["prepared_at"]
        assert sample["capture_kit"]
        assert sample["sequencing_method"]
        assert sample["sequenced_at"]
        assert sample["delivered_at"]

        assert sample["million_read_pairs"]
        assert sample["mapped_reads"]
        assert sample["target_coverage"]
        assert sample["target_completeness"]
        assert sample["duplicates"]
        assert sample["processing_time"]
        assert sample["analysis_sex"]

    assert delivery_data["pipeline"]
    assert delivery_data["pipeline_version"]
    assert delivery_data["genome_build"]


def is_similar_dicts(dict1, dict2):
    _is_similar = True

    for key in dict1.keys():
        _is_similar = _is_similar and is_similar_values(dict1.get(key), dict2.get(key))

    return _is_similar


def is_similar_lists(list1, list2):
    _is_similar = True

    if isinstance(list2, list):
        for value1, value2 in zip(list1, list2):
            _is_similar = _is_similar and is_similar_values(value1, value2)
    else:
        for value1 in list1:
            _is_similar = _is_similar and value1 in list2

    return _is_similar


def is_similar_values(value1, value2):

    if str(value1) == str(value2):
        return True

    if isinstance(value1, dict):
        return is_similar_dicts(value1, value2)

    if isinstance(value1, list):
        return is_similar_lists(value1, value2)

    if isinstance(value1, datetime.datetime):
        return str(value1.date()) == value2

    if is_float(value1):
        return round(float(value1), 1) == round(float(value2), 1)

    return False


def is_float(value):
    try:
        float(value)
        return True
    except (ValueError, TypeError):
        return False


def test_presentable_delivery_report_contains_delivery_data(report_api, case_id):
    # GIVEN data from an analysed case and an initialised report_api
    delivery_data = report_api._get_delivery_data(case_id=case_id)

    # WHEN creating delivery report
    presentable_data = report_api._make_data_presentable(delivery_data)

    # THEN
    # the delivery_report contains the delivery_data
    assert is_similar_dicts(delivery_data, presentable_data)


def dict_values_exists_in(a_dict: dict, a_target: str):

    all_exists = True

    for value in a_dict.values():
        all_exists = all_exists and value_exists_in(value, a_target)
    return all_exists


def value_exists_in(value, a_target: str):

    if isinstance(value, str):
        return value in a_target
    if isinstance(value, float):
        return str(round(value, 2)) in a_target or str(round(value, 1)) in a_target
    if isinstance(value, dict):
        return dict_values_exists_in(value, a_target)
    if isinstance(value, list):
        return list_values_exists_in(value, a_target)
    if isinstance(value, datetime.datetime):
        return str(value.date()) in a_target

    if str(value) in a_target:
        return True

    if isinstance(value, bool):
        return True

    return False


def list_values_exists_in(a_list: list, a_target: str):

    all_exists = True

    for value in a_list:
        all_exists = all_exists and value_exists_in(value, a_target)
    return all_exists


def test_create_delivery_report_contains_delivery_data(report_api):
    # GIVEN data from an analysed case and an initialised report_api
    case_id = "yellowhog"
    delivery_data = report_api._get_delivery_data(case_id=case_id)

    # WHEN creating delivery report
    delivery_report = report_api.create_delivery_report(case_id)

    # THEN
    # the delivery_report contains the delivery_data
    assert dict_values_exists_in(delivery_data, delivery_report)


def test_get_status_from_status_db(report_api):
    # GIVEN data from an analysed case and an initialised report_api

    # WHEN fetch_application_data_from_status_db
    samples = report_api._fetch_case_samples_from_status_db("yellowhog")

    # THEN the samples contain a status
    for sample in samples:
        assert sample["status"]


def test_incorporate_lims_methods(report_samples, report_api):
    # GIVEN data from an analysed case and an initialised report_api

    # WHEN fetch_and_add_lims_methods
    samples = report_samples
    report_api._incorporate_lims_methods(samples)

    # THEN
    # each sample has a property library_prep_method with a value
    for sample in samples:
        assert sample["prep_method"]

    # each sample has a property sequencing_method with a value
    for sample in samples:
        assert sample["sequencing_method"]


def test_render_delivery_report(report_api):
    # GIVEN proper qc data from an analysis exist
    delivery_data = report_api._get_delivery_data(case_id="yellowhog")
    report_data = report_api._make_data_presentable(delivery_data)

    # WHEN rendering a report from that data
    rendered_report = ReportAPI._render_delivery_report(report_data)

    # THEN a html report with certain data should have been rendered
    assert len(rendered_report) > 0


def test_create_delivery_report(report_api):
    # GIVEN initialized ReportAPI

    # WHEN rendering a report from that data
    created_report = report_api.create_delivery_report(case_id="yellowhog")

    # THEN a html report with certain data should have been rendered
    assert len(created_report) > 0


def test_create_delivery_report_file(report_api: ReportAPI):
    # GIVEN initialized ReportAPI

    # WHEN rendering a report from that data
    created_report_file = report_api.create_delivery_report_file(
        case_id="yellowhog", file_path=Path(".")
    )

    # THEN a html report with certain data should have been created on disk
    assert os.path.isfile(created_report_file.name)
    os.unlink(created_report_file.name)


def test_incorporate_coverage_data(report_api, report_samples):
    # GIVEN an initialised report_api and the chanjo_api does not return what we want
    report_api.chanjo._sample_coverage_returns_none = True
    samples = report_samples

    # WHEN failing to get latest trending data for a case
    report_api._incorporate_coverage_data(samples=samples, panels="dummyPanel")

    # THEN there should be a log entry about this
    for sample in samples:
        lims_id = sample["id"]
        assert not sample.get("target_coverage")
        assert not sample.get("target_completeness")
        lims_id_found_in_warnings = False
        for warning in report_api.log.get_warnings():
            lims_id_found_in_warnings = lims_id in warning or lims_id_found_in_warnings
        assert lims_id_found_in_warnings


def test_fetch_capture_kit_from_status_db(report_api):
    # GIVEN an initialised report_api and the db returns samples with capture kit

    # WHEN fetching status data
    samples = report_api._fetch_case_samples_from_status_db(case_id="yellowhog")

    # THEN the report data should have capture kit
    assert samples
    for sample in samples:
        assert sample.get("capture_kit") == "GMSmyeloid"


def test_data_analysis_kit_from_status_db(report_api, case_id):
    # GIVEN an initialised report_api and the db returns samples with data_analysis

    # WHEN fetching status data
    samples = report_api._fetch_case_samples_from_status_db(case_id=case_id)

    # THEN the report data should have capture kit
    assert samples
    for sample in samples:
        assert sample.get("capture_kit")


def test_get_application_data_from_status_db(report_samples, report_api):
    # GIVEN data from an analysed case and an initialised report_api

    # WHEN fetch_application_data_from_status_db
    samples = report_samples
    application_data = report_api._get_application_data_from_status_db(samples)

    # THEN
    # the generated data has a property apptags with a value
    assert application_data["applications"]


def test_get_application_data_from_status_db_all_accredited(report_samples, report_api):
    # GIVEN data from an analysed case and an initialised report_api
    report_api.store._application_accreditation = True

    # WHEN fetch_application_data_from_status_db
    samples = report_samples
    application_data = report_api._get_application_data_from_status_db(samples)

    # THEN the generated data has a property accredited with a value
    assert application_data["accredited"] is True


def test_get_application_data_from_status_db_none_accredited(report_samples, report_api):
    # GIVEN data from an analysed case and an initialised report_api
    report_api.store._application_accreditation = False

    # WHEN fetch_application_data_from_status_db
    samples = report_samples
    application_data = report_api._get_application_data_from_status_db(samples)

    # THEN the generated data has a property accredited with a value
    assert application_data["accredited"] is False


def test_render_accredited_delivery_report(report_api):
    # GIVEN proper qc data from an analysis exist with accredited application
    report_api.store._application_accreditation = True
    delivery_data = report_api._get_delivery_data(case_id="yellowhog")
    report_data = report_api._make_data_presentable(delivery_data)
    assert report_data["accredited"] is True

    # WHEN rendering a report from that data
    rendered_report = ReportAPI._render_delivery_report(report_data)

    # THEN a html report with swedac logo should have been rendered
    assert "SWEDAC logo" in rendered_report


def test_render_non_accredited_delivery_report(report_api):
    # GIVEN proper qc data from an analysis exist with non accredited application
    report_api.store._application_accreditation = False
    delivery_data = report_api._get_delivery_data(case_id="yellowhog")
    report_data = report_api._make_data_presentable(delivery_data)
    assert report_data["accredited"] is False

    # WHEN rendering a report from that data
    rendered_report = ReportAPI._render_delivery_report(report_data)

    # THEN a html report without swedac logo should have been rendered
    assert "SWEDAC logo" not in rendered_report


def test_get_delivery_data_not_accredited(report_api, report_store, case_id):
    # GIVEN an initialised report_api and a case ready for delivery report creation
    # GIVEN the case has samples that have at least one non accredited application

    # WHEN collecting delivery data for case
    case = report_store.family(case_id)
    assert case.links
    for link in case.links:
        link.sample.application_version.application.is_accredited = False

    delivery_data = report_api._get_delivery_data(case_id=case_id)

    # THEN the accreditation status int the delivery_data is false
    assert delivery_data["accredited"] is False


def test_get_delivery_data_accredited(report_api, report_store, case_id):
    # GIVEN an initialised report_api and a case ready for delivery report creation
    # GIVEN the case has samples that all have accredited application

    # WHEN collecting delivery data for case
    case = report_store.family(case_id)
    for link in case.links:
        link.sample.application_version.application.is_accredited = True
    delivery_data = report_api._get_delivery_data(case_id=case_id)

    # THEN the accreditation status int the delivery_data is true
    assert delivery_data["accredited"] is True



import logging
from pathlib import Path

import ruamel.yaml

from cg.apps.gt import GenotypeAPI
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.housekeeper.hk import models as housekeeper_models
from cg.apps.mip.parse_qcmetrics import parse_qcmetrics
from cg.store import models

LOG = logging.getLogger(__name__)


class UploadGenotypesAPI(object):
    def __init__(
        self,
        hk_api: HousekeeperAPI,
        gt_api: GenotypeAPI,
    ):
        LOG.info("Initializing UploadGenotypesAPI")
        self.hk = hk_api
        self.gt = gt_api

    def data(self, analysis_obj: models.Analysis) -> dict:
        """Fetch data about an analysis to load genotypes.

        Returns: dict on form

        {
            "bcf": path_to_bcf,
            "samples_sex": [
                "sample_id: {
                    "pedigree": "male",
                    "analysis": "male"
                    }
            ]
        }

        """
        case_id = analysis_obj.family.internal_id
        LOG.info("Fetching upload genotype data for %s", case_id)
        hk_version = self.hk.last_version(case_id)
        hk_bcf = self.get_bcf_file(hk_version)
        if hk_bcf is None:
            LOG.warning("unable to find GBCF for genotype upload")
            return None
        data = {"bcf": hk_bcf.full_path, "samples_sex": {}}
        qc_metrics_file = self.get_qcmetrics_file(hk_version)
        analysis_sexes = self.analysis_sex(qc_metrics_file)
        for link_obj in analysis_obj.family.links:
            sample_id = link_obj.sample.internal_id
            data["samples_sex"][sample_id] = {
                "pedigree": link_obj.sample.sex,
                "analysis": analysis_sexes[sample_id],
            }
        return data

    def analysis_sex(self, qc_metrics_file: Path) -> dict:
        """Fetch analysis sex for each sample of an analysis."""
        qcmetrics_data = self.get_parsed_qc_metrics_data(qc_metrics_file)
        data = {}
        for sample_id in qcmetrics_data:
            data[sample_id] = self.get_sample_predicted_sex(
                sample_id=sample_id, parsed_qcmetrics_data=qcmetrics_data
            )
        return data

    def get_bcf_file(self, hk_version_obj: housekeeper_models.Version) -> housekeeper_models.File:
        """Fetch a bcf file and return the file object"""

        bcf_file = self.hk.files(version=hk_version_obj.id, tags=["snv-gbcf"]).first()
        LOG.debug("Found bcf file %s", bcf_file.full_path)
        return bcf_file

    def get_qcmetrics_file(self, hk_version_obj: housekeeper_models.Version) -> Path:
        """Fetch a qc_metrics file and return the path"""
        hk_qcmetrics = self.hk.files(version=hk_version_obj.id, tags=["qcmetrics"]).first()
        LOG.debug("Found qc metrics file %s", hk_qcmetrics.full_path)
        return Path(hk_qcmetrics.full_path)

    @staticmethod
    def get_parsed_qc_metrics_data(qc_metrics: Path) -> dict:
        """Parse the information from a qc metrics file"""
        with qc_metrics.open() as in_stream:
            qcmetrics_raw = ruamel.yaml.safe_load(in_stream)
        return parse_qcmetrics(qcmetrics_raw)

    @staticmethod
    def get_sample_predicted_sex(sample_id: str, parsed_qcmetrics_data: dict) -> str:
        """Get the predicted sex for a sample"""
        if sample_id in parsed_qcmetrics_data:
            return parsed_qcmetrics_data[sample_id].get("predicted_sex", "unknown")
        return "unknown"

    def upload(self, data: dict, replace: bool = False):
        """Upload data about genotypes for a family of samples."""
        self.gt.upload(str(data["bcf"]), data["samples_sex"], force=replace)



"""Test the cli for uploading to scout"""
import logging
from pathlib import Path

from cg.cli.upload.scout import create_scout_load_config, scout
from cg.meta.upload.scout.scout_load_config import ScoutLoadConfig
from cg.meta.upload.scout.scoutapi import UploadScoutAPI
from cg.store import Store
from pydantic import ValidationError
from tests.meta.upload.scout.conftest import fixture_mip_analysis_hk_bundle_data


def check_log(caplog, string=None, warning=None):
    """Parse the log output"""
    found = False
    for _, level, message in caplog.record_tuples:
        if level == logging.WARNING and warning:
            found = True
        if string and string in message:
            found = True
    return found


def test_upload_with_load_config(
    base_context: dict,
    scout_load_config: Path,
    upload_scout_api: UploadScoutAPI,
    cli_runner,
    caplog,
):
    """Test to upload a case to scout using a load config"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a case that exists in status
    case_id = base_context["status_db"].families().first().internal_id
    tag_name = upload_scout_api.get_load_config_tag()
    # GIVEN  that a scout load config exists in housekeeper
    base_context["housekeeper_api"].add_file(
        path=str(scout_load_config), version_obj=None, tags=tag_name
    )
    load_config_file = base_context["housekeeper_api"].get_files(case_id, [tag_name])[0]
    assert load_config_file

    def case_exists_in_status(existing_case_id: str, store: Store):
        """Check if case exists in status database"""
        return store.families().first().internal_id == existing_case_id

    assert case_exists_in_status(case_id, base_context["status_db"])

    # WHEN invoking command to upload case to scout
    with caplog.at_level(logging.DEBUG):
        cli_runner.invoke(scout, [case_id], obj=base_context)

    # THEN assert that the case was loaded succesfully
    def case_loaded_succesfull(caplog):
        """Check output that case was loaded"""
        return check_log(caplog, string="Case loaded successfully to Scout")

    assert case_loaded_succesfull(caplog)

    # THEN assert that the load config was used
    def load_file_mentioned_in_result(caplog, load_config_file: Path):
        """Check output that load file is mentioned"""
        return f"uploaded to scout using load config {load_config_file}" in caplog.text

    assert load_file_mentioned_in_result(caplog, load_config_file.full_path)


def test_produce_load_config(
    base_context: dict, cli_runner, case_id: str, mip_analysis_hk_bundle_data: dict, helpers, caplog
):
    """Test create a scout load config with the scout upload api"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a singleton WGS case
    # GIVEN that the api generates a config
    # GIVEN a housekeeper instance with some bundle information
    hk_mock = base_context["housekeeper_api"]
    helpers.ensure_hk_bundle(hk_mock, mip_analysis_hk_bundle_data)

    # WHEN running cg upload scout -p <caseid>
    result = cli_runner.invoke(create_scout_load_config, [case_id, "--print"], obj=base_context)

    # THEN assert that the call was executed with success
    assert result.exit_code == 0
    # THEN there was some relevant output
    assert "owner" in result.output


def test_produce_load_config_no_delivery(
    base_context, cli_runner, analysis_family_single_case, scout_hk_bundle_data, helpers
):
    """Test to produce a load config without a delivery report"""
    # GIVEN a singleton WGS case

    base_context["scout_upload_api"].config.delivery_report = None
    # GIVEN a populated hk mock
    hk_mock = base_context["housekeeper_api"]
    helpers.ensure_hk_bundle(hk_mock, scout_hk_bundle_data)
    # GIVEN a housekeeper that does not return delivery files
    hk_mock.add_missing_tag("delivery-report")
    base_context["scout_upload_api"].housekeeper = hk_mock
    assert hk_mock.files(tags=["delivery-report"]).first() is None

    case_id = analysis_family_single_case["internal_id"]

    # WHEN running cg upload scout -p <caseid>
    result = cli_runner.invoke(scout, [case_id, "--print"], obj=base_context)

    # THEN assert the command succeded since delivery report is not mandatory
    assert result.exit_code == 0
    # THEN assert the output has some content
    assert result.output
    # THEN there is no delivery report in the output
    assert "'delivery_report'" not in result.output


def test_produce_load_config_missing_mandatory_file(
    base_context, cli_runner, case_id, scout_hk_bundle_data, helpers
):
    """Test to produce a load config when mandatory files are missing"""
    # GIVEN a singleton WGS case
    base_context["scout_upload_api"].missing_mandatory_field = True

    # GIVEN a populated hk mock
    hk_mock = base_context["housekeeper_api"]
    helpers.ensure_hk_bundle(hk_mock, scout_hk_bundle_data)
    # GIVEN a housekeeper that does not return mandatory files
    hk_mock.add_missing_tag("vcf-snv-clinical")
    base_context["scout_upload_api"].housekeeper = hk_mock
    assert hk_mock.files(tags=["vcf-snv-clinical"]).first() is None

    # WHEN running cg upload scout -p <caseid>
    result = cli_runner.invoke(scout, [case_id, "--print"], obj=base_context)

    # THEN assert the command failed since a mandatory file was missing
    assert result.exit_code != 0
    # THEN assert a FileNotFoundError was raised
    assert isinstance(result.exception, ValidationError)


def test_upload_scout_cli_file_exists(base_context, cli_runner, caplog, case_id):
    """Test to upload a case when the load config already exists"""
    # GIVEN a case_id where the case exists in status db with at least one analysis
    # GIVEN that the analysis is done and exists in tb
    assert isinstance(base_context["scout_upload_api"].generate_config("hej"), ScoutLoadConfig)
    # GIVEN that the upload file already exists
    base_context["scout_upload_api"].file_exists = True

    # WHEN uploading a case with the cli and printing the upload config
    result = cli_runner.invoke(scout, [case_id], obj=base_context)

    # THEN assert that the call exits without errors
    assert result.exit_code == 1

    # THEN assert that a warning is logged
    warned = check_log(caplog, warning=True)
    assert warned


def test_upload_scout_cli(base_context, cli_runner, case_id, scout_load_config):
    """Test to upload a case to scout using cg upload scout command"""
    # GIVEN a case_id where the case exists in status db with at least one analysis
    # GIVEN that the analysis is done and exists in tb
    tag_name = base_context["scout_upload_api"].get_load_config_tag()
    base_context["housekeeper_api"].add_file(
        path=scout_load_config, version_obj=None, tags=tag_name
    )
    # WHEN uploading a case with the cli and printing the upload config
    result = cli_runner.invoke(scout, [case_id], obj=base_context)

    # THEN assert that the call exits without errors
    assert result.exit_code == 0


def test_upload_scout_cli_print_console(
    base_context, cli_runner, case_id: str, scout_load_object: ScoutLoadConfig, caplog
):
    """Test to dry run a case upload"""
    caplog.set_level(logging.DEBUG)
    # GIVEN a case_id where the case exists in status db with at least one analysis
    # GIVEN that the analysis is done and exists in tb
    base_context["scout_upload_api"].config = scout_load_object
    # WHEN uploading a case with the cli and printing the upload config
    result = cli_runner.invoke(scout, [case_id, "--print"], obj=base_context)

    # THEN assert that the call exits without errors
    assert result.exit_code == 0

    assert case_id in caplog.text



"""Code for uploading coverage reports via CLI"""

import click
from cg.apps.coverage import ChanjoAPI
from cg.meta.upload.coverage import UploadCoverageApi

from .utils import suggest_cases_to_upload


@click.command()
@click.option("-r", "--re-upload", is_flag=True, help="re-upload existing analysis")
@click.argument("family_id", required=False)
@click.pass_context
def coverage(context, re_upload, family_id):
    """Upload coverage from an analysis to Chanjo."""

    click.echo(click.style("----------------- COVERAGE --------------------"))

    if not family_id:
        suggest_cases_to_upload(context)
        context.abort()

    chanjo_api = ChanjoAPI(context.obj)
    case_obj = context.obj["status_db"].family(family_id)
    api = UploadCoverageApi(context.obj["status_db"], context.obj["housekeeper_api"], chanjo_api)
    coverage_data = api.data(case_obj.analyses[0])
    api.upload(coverage_data, replace=re_upload)



"""Tests for the get cases functionality in ScoutAPI"""

from datetime import datetime
from cg.apps.scout.scoutapi import ScoutAPI


def test_get_cases_no_case(scout_api: ScoutAPI, case_id: str):
    """Test to get cases based on case_id when there are no matches"""
    # GIVEN a scout api and a process that returns no std output
    assert scout_api.process.stdout == ""

    # WHEN querying for cases
    result = scout_api.get_cases(case_id=case_id)

    # THEN assert that the empty list is returned since there where cases in scout
    assert result == []


def test_get_cases_one_case(scout_api: ScoutAPI, case_id: str, export_cases_output: str):
    """Test to get get a case based on case_id when there is a case in scout"""
    scout_api.process.set_stdout(export_cases_output)
    # GIVEN a scout api and a process that returns some relevant input
    assert scout_api.process.stdout == export_cases_output

    # WHEN querying for cases
    result = scout_api.get_cases(case_id=case_id)

    # THEN assert that the output is a list
    assert isinstance(result, list)
    # THEN assert that there was one case in the list
    assert len(result) == 1
    case_data = result[0]

    # THEN assert that the case has a _id
    assert case_data.id
    # THEN assert that the analysis date is a datetime object
    assert isinstance(case_data.analysis_date, datetime)



"""Fixtures for compress api tests"""
import copy
import json
import os
from datetime import datetime
from pathlib import Path

import pytest
from cg.apps.crunchy import CrunchyAPI
from cg.meta.compress import CompressAPI


class CompressionData:
    """Class to hold some data files"""

    def __init__(self, **kwargs):
        self.spring_path = kwargs["spring_path"]
        self.pending_path = kwargs["pending_path"]
        self.spring_metadata_path = kwargs["spring_metadata_path"]
        self.fastq_first = kwargs["fastq_first"]
        self.fastq_second = kwargs["fastq_second"]

    @property
    def spring_file(self):
        """Return the path to an existing spring file"""
        self.spring_path.touch()
        return self.spring_path

    @property
    def pending_file(self):
        """Return the path to an existing pending file"""
        self.pending_path.touch()
        return self.pending_path

    @staticmethod
    def _spring_metadata(fastq_first, fastq_second, spring_path, updated=False, date=None):
        """Return metada information"""
        metadata = [
            {
                "path": str(fastq_first.resolve()),
                "file": "first_read",
                "checksum": "checksum_first_read",
                "algorithm": "sha256",
            },
            {
                "path": str(fastq_second.resolve()),
                "file": "second_read",
                "checksum": "checksum_second_read",
                "algorithm": "sha256",
            },
            {"path": str(spring_path.resolve()), "file": "spring"},
        ]
        if updated:
            for file_info in metadata:
                file_info["updated"] = date or "2020-01-21"
        return metadata

    @property
    def spring_metadata_file(self):
        """Return the path to an existing spring metadata file"""

        spring_metadata = CompressionData._spring_metadata(
            self.fastq_first, self.fastq_second, self.spring_path
        )
        with open(self.spring_metadata_path, "w") as outfile:
            outfile.write(json.dumps(spring_metadata))

        return self.spring_metadata_path

    @property
    def updated_spring_metadata_file(self):
        """Return the path to an existing spring metadata file"""

        spring_metadata = CompressionData._spring_metadata(
            self.fastq_first, self.fastq_second, self.spring_path, True
        )
        with open(self.spring_metadata_path, "w") as outfile:
            outfile.write(json.dumps(spring_metadata))

        return self.spring_metadata_path

    @staticmethod
    def make_old(file_path):
        """"Convert the modifying date so that the file looks old"""
        # Convert the date to a float
        before_timestamp = datetime.timestamp(datetime(2020, 1, 1))
        # Update the utime so file looks old
        os.utime(file_path, (before_timestamp, before_timestamp))

    @property
    def fastq_first_file(self, old: bool = True):
        """Return the path to an existing spring metadata file"""
        self.fastq_first.touch()
        if old:
            CompressionData.make_old(self.fastq_first)
        return self.fastq_first

    @property
    def fastq_second_file(self, old: bool = True):
        """Return the path to an existing fastq file"""
        self.fastq_second.touch()
        if old:
            CompressionData.make_old(self.fastq_second)
        return self.fastq_second


@pytest.fixture(scope="function", name="compression_files")
def fixture_compression_files(compression_object):
    """Return a small class with compression files"""
    return CompressionData(
        spring_path=compression_object.spring_path,
        spring_metadata_path=compression_object.spring_metadata_path,
        fastq_first=compression_object.fastq_first,
        fastq_second=compression_object.fastq_second,
        pending_path=compression_object.pending_path,
    )


@pytest.fixture(scope="function", name="real_crunchy_api")
def fixture_real_crunchy_api(crunchy_config_dict):
    """crunchy api fixture"""

    yield CrunchyAPI(crunchy_config_dict)


@pytest.fixture(scope="function", name="compress_api")
def fixture_compress_api(real_crunchy_api, housekeeper_api):
    """compress api fixture"""
    hk_api = housekeeper_api
    _api = CompressAPI(crunchy_api=real_crunchy_api, hk_api=hk_api)
    yield _api


@pytest.fixture(scope="function", name="populated_compress_fastq_api")
def fixture_populated_compress_fastq_api(compress_api, compress_hk_fastq_bundle, helpers):
    """Populated compress api fixture"""
    helpers.ensure_hk_bundle(compress_api.hk_api, compress_hk_fastq_bundle)

    return compress_api


@pytest.fixture(scope="function", name="populated_decompress_spring_api")
def fixture_populated_decompress_spring_api(compress_api, decompress_hk_spring_bundle, helpers):
    """Populated compress api fixture"""
    helpers.ensure_hk_bundle(compress_api.hk_api, decompress_hk_spring_bundle)

    return compress_api


@pytest.fixture(scope="function", name="sample")
def fixture_sample():
    """Return the sample id for first sample"""
    return "sample_1"


@pytest.fixture(scope="function", name="spring_path")
def fixture_spring_path(compression_object) -> Path:
    """Return the path to a non existing spring file"""
    return compression_object.spring_path


@pytest.fixture(scope="function", name="spring_metadata_path")
def fixture_spring_metadata_path(compression_object) -> Path:
    """Return the path to a non existing spring metadata file"""
    return compression_object.spring_metadata_path


@pytest.fixture(scope="function", name="fastq_flag_path")
def fixture_fastq_flag_path(spring_metadata_path) -> Path:
    """Return the path to a non existing fastq flag file"""
    return spring_metadata_path


@pytest.fixture(scope="function", name="fastq_flag_file")
def fixture_fastq_flag_file(spring_metadata_path) -> Path:
    """Return the path to an existing fastq flag file"""
    spring_metadata_path.touch()
    return spring_metadata_path


@pytest.fixture(scope="function", name="spring_file")
def fixture_spring_file(spring_path) -> Path:
    """Return the path to an existing spring file"""
    spring_path.touch()
    return spring_path


@pytest.fixture(scope="function", name="multi_linked_file")
def fixture_multi_linked_file(spring_path, project_dir) -> Path:
    """Return the path to an existing file with two links"""
    first_link = project_dir / "link-1"
    os.link(spring_path, first_link)

    return spring_path


@pytest.fixture(scope="function", name="fastq_paths")
def fixture_fastq_paths(compression_object):
    """Fixture for temporary fastq-files

    Create fastq paths and return a dictionary with them
    """
    return {
        "fastq_first_path": compression_object.fastq_first,
        "fastq_second_path": compression_object.fastq_second,
    }


@pytest.fixture(scope="function", name="fastq_files")
def fixture_fastq_files(fastq_paths):
    """Fixture for temporary fastq-files

    Create fastq files and return a dictionary with them
    """
    fastq_first_file = fastq_paths["fastq_first_path"]
    fastq_second_file = fastq_paths["fastq_second_path"]
    fastq_first_file.touch()
    fastq_second_file.touch()

    return fastq_paths


# Bundle fixtures


@pytest.fixture(scope="function", name="decompress_hk_spring_bundle")
def fixture_decompress_hk_spring_bundle(
    sample_hk_bundle_no_files, spring_file, fastq_flag_file, sample
):
    """Create a complete bundle mock for testing decompression

    This bundle contains a spring file and a spring metadata file
    """
    hk_bundle_data = copy.deepcopy(sample_hk_bundle_no_files)

    spring_file_info = {"path": str(spring_file), "archive": False, "tags": [sample, "spring"]}
    spring_meta_info = {
        "path": str(fastq_flag_file),
        "archive": False,
        "tags": [sample, "spring-metadata"],
    }

    hk_bundle_data["files"].append(spring_file_info)
    hk_bundle_data["files"].append(spring_meta_info)

    return hk_bundle_data



"""
    Tests for loqusdbAPI
"""

import json
import subprocess

import pytest
from cg.apps.loqus import LoqusdbAPI
from cg.exc import CaseNotFoundError


def test_instatiate(loqus_config):

    """Test to instantiate a loqusdb api"""
    # GIVEN a loqusdb api with some configs

    # WHEN instantiating a loqusdb api
    loqusdb = LoqusdbAPI(loqus_config)

    # THEN assert that the adapter was properly instantiated
    assert loqusdb.loqusdb_config == loqus_config["loqusdb"]["config_path"]
    assert loqusdb.loqusdb_binary == loqus_config["loqusdb"]["binary_path"]

    # WHEN instatiating with analysis_type argument as 'wes'
    loqusdb = LoqusdbAPI(loqus_config, analysis_type="wes")

    # THEN assert that the adapter was properly instantiated
    assert loqusdb.loqusdb_config == loqus_config["loqusdb-wes"]["config_path"]
    assert loqusdb.loqusdb_binary == loqus_config["loqusdb-wes"]["binary_path"]


def test_get_case(loqusdbapi, loqusdb_case_output):

    """Test to get a case via the api"""
    # GIVEN a loqusdb api
    case_id = "a_case"
    loqusdbapi.process.stdout = loqusdb_case_output.decode("utf-8").rstrip()
    # WHEN fetching a case with the adapter
    case_obj = loqusdbapi.get_case(case_id)
    # THEN assert that the correct case id is returned
    assert case_obj["case_id"] == case_id


def test_get_case_non_existing(loqusdbapi):

    """Test to get a case via the api"""

    # GIVEN a loqusdb api and a case id
    case_id = "a_case"

    # WHEN loqusdb output is empty string

    # THEN CaseNotFoundError should be raised
    with pytest.raises(CaseNotFoundError):
        loqusdbapi.get_case(case_id)


def test_get_case_command_fail(loqusdbapi_exception):

    """Test to see if exception is raised"""
    # GIVEN a loqusdb api and a case id
    loqusdbapi = loqusdbapi_exception
    case_id = "a_case"
    # WHEN an error occurs during fetching a case with the adapter

    # THEN assert that the error is raised
    with pytest.raises(subprocess.CalledProcessError):
        loqusdbapi.get_case(case_id)


def test_get_duplicate(loqusdbapi, loqusdb_duplicate_output):
    """Test when duplicate exists"""

    # GIVEN a loqusdb api and a duplicate output
    loqusdbapi.process.stdout = loqusdb_duplicate_output.decode("utf-8")
    dup_json = json.loads(loqusdb_duplicate_output.decode("utf-8"))

    # WHEN a duplicate exists
    duplicate = loqusdbapi.get_duplicate("vcf_path")

    # THEN assert that the duplicate individual is returned
    assert dup_json == duplicate


def test_get_duplicate_non_existing(loqusdbapi):
    """Test when no duplicate exists"""

    # GIVEN a loqusdb api

    # WHEN duplicate does not exist
    duplicate = loqusdbapi.get_duplicate("vcf_path")

    # THEN assert that an empty dict is returned
    assert duplicate == {}


def test_load(loqusdbapi, loqusdb_output):

    """Test to load a case"""
    # GIVEN a loqusdb api and some info about a case
    case_id = "test"
    ped_path = "a ped path"
    vcf_path = "a vcf path"
    vcf_sv_path = "a sv_vcf path"
    gbcf_path = "a bcf path"

    # WHEN uploading a case with 15 variants to loqusdb
    loqusdbapi.process.stderr = loqusdb_output.decode("utf-8")

    data = loqusdbapi.load(case_id, ped_path, vcf_path, vcf_sv_path, gbcf_path)

    # THEN assert that the number of variants is 15

    assert data["variants"] == 15


def test_repr_string(loqus_config):

    """Test __repr__ of loqusdbAPI"""

    loqusdb = LoqusdbAPI(loqus_config)

    repr_string = repr(loqusdb)

    correct_string = (
        f"LoqusdbAPI(binary={loqus_config['loqusdb']['binary_path']},"
        f"config={loqus_config['loqusdb']['config_path']})"
    )

    assert repr_string == correct_string



import copy
import datetime as dt
from pathlib import Path

import pytest

from cg.apps.crunchy import CrunchyAPI
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.tb import TrailblazerAPI
from cg.apps.tb.models import TrailblazerAnalysis as tb_Analysis
from cg.constants import Pipeline
from cg.meta.compress.compress import CompressAPI
from cg.meta.workflow.mip import MipAnalysisAPI
from cg.meta.workflow.prepare_fastq import PrepareFastqAPI
from cg.store import Store
from ruamel.yaml import YAML
from tests.mocks.limsmock import MockLimsAPI


@pytest.fixture(name="mip_lims")
def fixture_mip_lims() -> MockLimsAPI:
    mip_lims = MockLimsAPI({})
    return mip_lims


@pytest.fixture(name="mock_mip_script")
def fixture_mock_mip_script() -> str:
    return "echo"


@pytest.fixture
def mock_root_folder(project_dir: Path) -> str:
    return Path(project_dir, "cases").as_posix()


@pytest.fixture(name="mip_case_ids")
def fixture_mip_case_ids() -> dict:
    """Dictionary of case ids, connected samples, their name and if they should fail (textbook or not)"""

    case_ids = {
        "yellowhog": {
            "textbook": True,
            "name": "F0000001",
            "internal_id": "ACC00000",
        },
        "purplesnail": {
            "textbook": False,
            "name": "F0000003",
            "internal_id": "ACC00000",
        },
        "bluezebra": {
            "textbook": True,
            "name": "F0000002",
            "internal_id": "ACC00000",
        },
    }

    return case_ids


@pytest.fixture(name="mip_case_dirs")
def fixture_mip_case_dirs(mip_case_ids: dict, project_dir: Path) -> dict:
    """Create case directories and return a dictionary of the tmpdir paths"""

    mip_case_dirs = {}

    for case in mip_case_ids:
        case_dir = Path(project_dir / "cases" / case)
        case_dir.mkdir(exist_ok=True, parents=True)
        mip_case_dirs[case] = case_dir

    return mip_case_dirs


@pytest.fixture(name="mip_deliverables")
def fixture_mip_deliverables(
    mip_case_ids: dict,
    mip_case_dirs: dict,
    mip_deliverables_file: Path,
) -> dict:
    """Create deliverables for mip store testing"""

    yaml = YAML()

    # Create a dict of paths to the deliverables to be used in later stages
    deliverables_paths = {}

    # Load general deliverables file
    with open(mip_deliverables_file, "r") as mip_dna_deliverables:
        mip_deliverables = yaml.load(mip_dna_deliverables)

    for case in mip_case_ids:
        if mip_case_ids[case]["textbook"]:
            case_specific_deliverables = copy.deepcopy(mip_deliverables)

            for file in case_specific_deliverables["files"]:
                if file["path_index"]:
                    file["path_index"] = str(mip_case_dirs[case] / Path(file["path_index"]).name)
                    # Touch the tmp file
                    open(file["path_index"], "a").close()
                file["path"] = str(mip_case_dirs[case] / Path(file["path"]).name)
                # Touch the tmp file
                open(file["path"], "a").close()

            case_deliverables_path = mip_case_dirs[case] / f"{case}_deliverables.yaml"
            with open(case_deliverables_path, "w") as fh:
                yaml.dump(case_specific_deliverables, fh)
            deliverables_paths[case] = case_deliverables_path

        elif not mip_case_ids[case]["textbook"]:
            case_deliverables_path = mip_case_dirs[case] / f"{case}_deliverables.yaml"
            with open(case_deliverables_path, "w") as fh:
                yaml.dump({"files": []}, fh)
            deliverables_paths[case] = case_deliverables_path

    return deliverables_paths


@pytest.fixture(name="mip_qc_sample_info")
def fixture_mip_qc_sample_info(mip_case_ids: dict, mip_case_dirs: dict) -> dict:
    """Create qc sample info files for cases"""

    yaml = YAML()

    # Store paths in a list to be used in other stages

    qc_sample_info = {}

    for case in mip_case_ids:
        sample_info_data = {
            "date": dt.datetime.now(),
            "analysisrunstatus": "finished",
            "analysis_date": dt.datetime.now(),
            "case": case,
            "mip_version": "latest",
        }

        sample_info_path = mip_case_dirs[case] / f"{case}_qc_sample_info.yaml"
        qc_sample_info[case] = sample_info_path
        with open(sample_info_path, "w") as fh:
            yaml.dump(sample_info_data, fh)

    return qc_sample_info


@pytest.fixture(name="mip_configs")
def fixture_mip_configs(
    mip_case_ids: dict, mip_case_dirs: dict, mip_deliverables: dict, mip_qc_sample_info: dict
) -> dict:
    """Create config files for mip case"""

    yaml = YAML()

    # Storing paths in a list to be returned and used in later stages.

    config_dict = {}

    for case in mip_case_ids:
        config = {
            "email": "fake_email@notscilifelab.com",
            "case_id": case,
            "samples": "dummy_samples",
            "is_dryrun": False,
            "outdata_dir": str(mip_case_dirs[case]),
            "priority": "medium",
            "sampleinfo_path": str(mip_qc_sample_info[case]),
            "sample_info_file": str(mip_qc_sample_info[case]),
            "analysis_type": {mip_case_ids[case]["internal_id"]: "wgs"},
            "slurm_quality_of_service": "medium",
            "store_file": str(mip_deliverables[case]),
        }
        analysis_path = Path(mip_case_dirs[case] / "analysis")
        Path(analysis_path).mkdir(parents=True, exist_ok=True)
        config_path = analysis_path / f"{case}_config.yaml"
        config_dict[case] = config_path
        with open(config_path, "w") as fh:
            yaml.dump(config, fh)

    return config_dict


@pytest.fixture(name="_store")
def fixture_store(base_store: Store, mip_case_ids: dict, helpers) -> Store:
    """Create and populate temporary db with test cases for mip"""

    _store = base_store

    # Add apptag to db

    helpers.ensure_application_version(store=_store, application_tag="WGSA", application_type="wgs")

    # Add sample, cases and relationships to db

    for case_id in mip_case_ids:
        case_obj = helpers.add_case(
            store=_store,
            internal_id=case_id,
            case_id=mip_case_ids[case_id]["name"],
        )
        sample = helpers.add_sample(
            store=_store,
            sample=mip_case_ids[case_id]["internal_id"],
            data_analysis=Pipeline.MIP_DNA,
            customer_name="cust000",
            application_tag="WGSA",
            application_type="wgs",
            gender="unknown",
        )
        helpers.add_relationship(store=_store, sample=sample, case=case_obj, status="affected")

    return _store


@pytest.fixture(name="empty_housekeeper_api")
def fixture_empty_housekeeper_api(real_housekeeper_api: HousekeeperAPI) -> HousekeeperAPI:
    """Empty housekeeper for testing store in mip workflow"""

    _housekeeper_api = real_housekeeper_api

    return _housekeeper_api


@pytest.fixture(name="populated_mip_tb_api")
def fixture_populated_mip_tb_api(
    trailblazer_api: TrailblazerAPI, mip_case_ids: dict, mip_configs: dict
) -> TrailblazerAPI:
    """Trailblazer api filled with mip cases"""

    _tb_api = trailblazer_api

    for case in mip_case_ids:
        _tb_api.add_commit(
            tb_Analysis(
                family=case, status="completed", deleted=False, config_path=str(mip_configs[case])
            )
        )

    return _tb_api


@pytest.fixture(name="mip_rna_conda_env_name")
def fixture_mip_rna_conda_env_name() -> str:
    return "S_mip_rd-rna"


@pytest.fixture(name="mip_dna_conda_env_name")
def fixture_mip_dna_conda_env_name() -> str:
    return "S_mip_rd-dna"


@pytest.fixture(name="mip_dna_pipeline")
def fixture_mip_dna_pipeline() -> str:
    return "analyse rd_dna"


@pytest.fixture(name="mip_rna_pipeline")
def fixture_mip_rna_pipeline() -> str:
    return "analyse rd_rna"


@pytest.fixture(scope="function", name="analysis_store_rna_case")
def fixture_analysis_store_rna_case(
    base_store: Store, analysis_family_single_case: dict, apptag_rna: str, helpers
) -> Store:
    """Setup a store instance with a single ind RNA case for testing analysis API"""
    analysis_family_single_case["data_analysis"] = str(Pipeline.MIP_RNA)
    helpers.ensure_case_from_dict(
        base_store, case_info=analysis_family_single_case, app_tag=apptag_rna
    )

    yield base_store


@pytest.fixture(name="rna_mip_context")
def fixture_rna_mip_context(
    analysis_store_rna_case: Store,
    trailblazer_api: TrailblazerAPI,
    housekeeper_api: HousekeeperAPI,
    mip_lims: MockLimsAPI,
    mock_root_folder: str,
    mock_mip_script: str,
    mip_rna_conda_env_name: str,
    mip_dna_conda_env_name: str,
    mip_rna_pipeline: str,
):
    rna_mip_context = {
        "rna_api": MipAnalysisAPI(
            db=analysis_store_rna_case,
            hk_api=housekeeper_api,
            tb_api=trailblazer_api,
            scout_api="scout_api",
            lims_api=mip_lims,
            script=mock_mip_script,
            pipeline=mip_rna_pipeline,
            conda_env=mip_rna_conda_env_name,
            root=mock_root_folder,
        ),
        "mip-rd-rna": {
            "conda_env": mip_rna_conda_env_name,
            "mip_config": "config.yaml",
            "pipeline": mip_rna_pipeline,
            "root": mock_root_folder,
            "script": mock_mip_script,
        },
    }
    return rna_mip_context


@pytest.fixture(name="crunchy_config")
def fixture_crunchy_config(crunchy_config_dict) -> dict:
    """Returns a config for CrunchyAPI"""
    return crunchy_config_dict


@pytest.fixture(name="crunchy")
def fixture_crunchy(crunchy_config_dict) -> CrunchyAPI:
    """Returns CrunchyAPI"""
    crunchy_api = CrunchyAPI(crunchy_config_dict)
    return crunchy_api


@pytest.fixture(name="compress")
def fixture_compress(housekeeper_api, crunchy) -> CompressAPI:
    """Returns CompressAPI"""
    compress_api = CompressAPI(hk_api=housekeeper_api, crunchy_api=crunchy)
    return compress_api


@pytest.fixture(name="dna_mip_context")
def fixture_dna_mip_context(
    analysis_store_single_case: Store,
    trailblazer_api: TrailblazerAPI,
    housekeeper_api: HousekeeperAPI,
    crunchy: CrunchyAPI,
    compress: CompressAPI,
    crunchy_config: dict,
    mip_lims: MockLimsAPI,
    mock_root_folder: str,
    mock_mip_script: str,
    mip_rna_conda_env_name: str,
    mip_dna_conda_env_name: str,
    mip_dna_pipeline: str,
):
    dna_mip_context = {
        "dna_api": MipAnalysisAPI(
            db=analysis_store_single_case,
            hk_api=housekeeper_api,
            tb_api=trailblazer_api,
            scout_api="scout_api",
            lims_api=mip_lims,
            script=mock_mip_script,
            pipeline=mip_dna_pipeline,
            conda_env=mip_dna_conda_env_name,
            root=mock_root_folder,
        ),
        "crunchy_api": CrunchyAPI(crunchy_config),
        "compress_api": CompressAPI(hk_api=housekeeper_api, crunchy_api=crunchy),
        "prepare_fastq_api": PrepareFastqAPI(
            store=analysis_store_single_case, compress_api=compress
        ),
        "mip-rd-dna": {
            "conda_env": mip_dna_conda_env_name,
            "mip_config": "config.yaml",
            "pipeline": mip_dna_pipeline,
            "root": mock_root_folder,
            "script": mock_mip_script,
        },
    }
    return dna_mip_context


@pytest.fixture(name="mip_store_context")
def mip_store_context(
    trailblazer_api, _store: Store, empty_housekeeper_api: HousekeeperAPI, mock_root_folder: Path
) -> dict:
    """Create a context to be used in testing mip store, this should be fused with mip_context above at later stages"""
    return {
        "mip_api": MipAnalysisAPI(
            db=_store,
            tb_api=trailblazer_api,
            hk_api=empty_housekeeper_api,
            scout_api=None,
            lims_api=None,
            script="None",
            pipeline="None",
            conda_env="None",
            root=mock_root_folder,
        ),
        "trailblazer_api": trailblazer_api,
        "housekeeper_api": empty_housekeeper_api,
        "status_db": _store,
    }
